package org.andorp.testkeys.stage1;

import junit.framework.Assert;

import org.andorp.testkeys.Context;
import org.andorp.testkeys.Keyword;
import org.andorp.testkeys.KeywordException;
import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.KeywordRepr;
import org.andorp.testkeys.RollbackException;
import org.andorp.testkeys.container.Standard;
import org.andorp.testkeys.story.ContainerException;
import org.andorp.testkeys.story.Story;
import org.andorp.testkeys.story.StringStoryBuilder;
import org.andorp.testkeys.story.TestFactory;
import org.andorp.testkeys.test.Result;
import org.andorp.testkeys.test.Runner;
import org.testng.annotations.Test;

public final class TestCompositeKeyword {

	private static class SimpleStep implements Keyword {

		private final int n;
		
		public SimpleStep(int _n) {
			n = _n;
		}
		
		public KeywordName name() {
			return new KeywordName("Simple Step #" + n);
		}

		public boolean step(Context c) throws KeywordException {
			System.out.println("Simple step" + n);
			return true;
		}

		public void rollback() throws RollbackException {
			System.out.println("Simple step rollback" + n);
		}
		
	}
	
	private static class FailureStep implements Keyword {

		private final int n;
		
		public FailureStep(int _n) {
			n = _n;
		}
		
		public KeywordName name() {
			return new KeywordName("Failure Step #" + n);
		}

		public boolean step(Context c) throws KeywordException {
			System.out.println("Failure step #" + n);
			return false;
		}

		public void rollback() throws RollbackException {
			System.out.println("Failure step rollback #" + n);
		}
	}
	
	private static class ExceptionStep implements Keyword {
		
		private final int n;
		
		public ExceptionStep(int _n) {
			n = _n;
		}
		
		
		public KeywordName name() {
			return new KeywordName("Exception Step #" + n);
		}

		public boolean step(Context c) throws KeywordException {
			throw new RuntimeException("Exception Step #" + n);
		}

		public void rollback() throws RollbackException {
			System.out.println("Exception step rollback #" + n);
		}
		
	}
	
	private static class RollbackExceptionStep implements Keyword {
		
		private final int n;
		
		public RollbackExceptionStep(int _n) {
			n = _n;
		}
		
		
		public KeywordName name() {
			return new KeywordName("Rollback Exception Step #" + n);
		}

		public boolean step(Context c) throws KeywordException {
			System.out.println("Rollback Exception Step #" + n);
			return true;
		}

		public void rollback() throws RollbackException {
			throw new RollbackException("");
		}
		
	}	
	
	@Test()
	public void threeSimpleSteps() throws ContainerException, RollbackException {
		final SimpleStep s1 = new SimpleStep(1);
		final SimpleStep s2 = new SimpleStep(2);
		final SimpleStep s3 = new SimpleStep(3);
		
		final Standard container = new Standard();
		final TestFactory testFactory = new TestFactory(container);
		
		container.register(s1);
		container.register(s2);
		container.register(s3);
		
		final StringStoryBuilder b = new StringStoryBuilder();
		b.add("Simple Step #1");
		b.add("Simple Step #2");
		b.add("Simple Step #3");
		
		final Story story = b.build("Story");
		final KeywordRepr keyword = testFactory.createTestCase(story);
		
		final Runner r = new Runner(); 
		
		final Result result = r.run(Context.EmptyContext(), keyword);
		Assert.assertEquals(Result.PASSED, result);
	}
	
	@Test()
	public void secondStepFail() throws ContainerException, RollbackException {
		final SimpleStep s1 = new SimpleStep(1);
		final FailureStep s2 = new FailureStep(2);
		final SimpleStep s3 = new SimpleStep(3);
		
		final Standard container = new Standard();
		final TestFactory testFactory = new TestFactory(container);
		
		container.register(s1);
		container.register(s2);
		container.register(s3);
		
		final StringStoryBuilder b = new StringStoryBuilder();
		b.add("Simple Step #1");
		b.add("Failure Step #2");
		b.add("Simple Step #3");
		
		final Story story = b.build("Story");
		final KeywordRepr keyword = testFactory.createTestCase(story);
		
		final Runner r = new Runner(); 
		
		final Result result = r.run(Context.EmptyContext(), keyword);
		Assert.assertEquals(Result.FAILED, result);
	}

	@Test()
	public void exceptionStepFail() throws ContainerException, RollbackException {
		final SimpleStep s1 = new SimpleStep(1);
		final ExceptionStep s2 = new ExceptionStep(2);
		final SimpleStep s3 = new SimpleStep(3);
		
		final Standard container = new Standard();
		final TestFactory testFactory = new TestFactory(container);
		
		container.register(s1);
		container.register(s2);
		container.register(s3);
		
		final StringStoryBuilder b = new StringStoryBuilder();
		b.add("Simple Step #1");
		b.add("Exception Step #2");
		b.add("Simple Step #3");
		
		final Story story = b.build("Story");
		final KeywordRepr keyword = testFactory.createTestCase(story);
		
		final Runner r = new Runner(); 
		
		final Result result = r.run(Context.EmptyContext(), keyword);
		Assert.assertEquals(Result.FAILED, result);
	}
	
	@Test(expectedExceptions = RollbackException.class)
	public void rollbackExceptionStepFail() throws ContainerException, RollbackException {
		final RollbackExceptionStep s1 = new RollbackExceptionStep(1);
		final ExceptionStep s2 = new ExceptionStep(2);
		final SimpleStep s3 = new SimpleStep(3);
		
		final Standard container = new Standard();
		final TestFactory testFactory = new TestFactory(container);
		
		container.register(s1);
		container.register(s2);
		container.register(s3);
		
		final StringStoryBuilder b = new StringStoryBuilder();
		b.add("Rollback Exception Step #1");
		b.add("Exception Step #2");
		b.add("Simple Step #3");
		
		final Story story = b.build("Story");
		final KeywordRepr keyword = testFactory.createTestCase(story);
		
		final Runner r = new Runner(); 
		
		r.run(Context.EmptyContext(), keyword);
	}
	
}
